#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/poll.h>
#include <sys/time.h>

#include "nsu.h"

struct timeval		tv_start;

unsigned long get_usec()
{
	struct timeval		tv;

	gettimeofday(&tv, NULL);

	return (tv.tv_sec - tv_start.tv_sec) * 1000 * 1000 + 
		(tv.tv_usec - tv_start.tv_usec);
}

int main(int argc, char *argv[])
{
	if (argc < 3) {
		fprintf(stderr, "Too few arguments\n");
		exit(1);
	}

	char			*targ_ipstr = argv[1];
	unsigned long		st;
	unsigned long		dt = strtol(argv[2], NULL, 10);
	struct sockaddr_in	host_addr;
	struct sockaddr_in	targ_addr;
	struct sockaddr_in	recv_addr;
	socklen_t		recv_len;
	int			sock;
	struct pollfd		ufds;
	int			ret;
	struct nsu_packet	packet;
	int			pts = 0;
	char			ipstr[32];

	sock = socket(AF_INET, SOCK_DGRAM, 0);

	if (sock < 0) {
		fprintf(stderr, "socket failed: %s\n", strerror(errno));
		exit(1);
	}

	host_addr.sin_family = AF_INET;
	host_addr.sin_addr.s_addr = INADDR_ANY;
	host_addr.sin_port = htons(NSU_PLANT_PORT);

	if (bind(sock, (struct sockaddr *) &host_addr,
				sizeof(struct sockaddr_in)) < 0) {
		fprintf(stderr, "bind failed: %s\n", strerror(errno));
		exit(1);
	}

	targ_addr.sin_family = AF_INET;
	targ_addr.sin_addr.s_addr = INADDR_ANY;
	targ_addr.sin_port = htons(NSU_CONTROLLER_PORT);

	if (inet_pton(AF_INET, targ_ipstr,
				&targ_addr.sin_addr.s_addr) < 0) {
		fprintf(stderr, "inet_pton failed: %s\n", strerror(errno));
		exit(1);
	}

	ufds.fd = sock;
	ufds.events = POLLIN;

	gettimeofday(&tv_start, NULL);

	do {
		packet.timestamp = htonl(pts++);

		ret = sendto(sock, &packet, sizeof(struct nsu_packet), 0,
				(struct sockaddr *) &targ_addr,
				sizeof(struct sockaddr_in));

		if (ret < 0) {
			fprintf(stderr, "send failed: %s\n", strerror(errno));
		}

		if (inet_ntop(AF_INET, &targ_addr.sin_addr.s_addr,
					ipstr, 32) == NULL) {
			fprintf(stderr, "inet_ntop failed: %s\n", strerror(errno));
		}

		printf("usec %lu send (%s:%hu) timestamp %i \n",
				get_usec(),
				ipstr, ntohs(targ_addr.sin_port),
				ntohl(packet.timestamp));
		fflush(stdout);

		st = get_usec();

		do {
			ret = poll(&ufds, 1, (pts * dt - (st)) / 1000);

			if (ret < 0) {
				fprintf(stderr, "poll failed: %s\n", strerror(errno));
			}
			else if (ret > 0) {

				recv_len = sizeof(struct sockaddr_in);
				ret = recvfrom(sock, &packet, sizeof(struct nsu_packet), 0,
						(struct sockaddr *) &recv_addr, &recv_len);

				if (ret < 0) {
					fprintf(stderr, "recv failed: %s\n", strerror(errno));
				}

				if (inet_ntop(AF_INET, &recv_addr.sin_addr.s_addr,
							ipstr, 32) == NULL) {
					fprintf(stderr, "inet_ntop failed: %s\n", strerror(errno));
				}

				printf("usec %lu recv (%s:%hu) timestamp %i \n",
						get_usec(),
						ipstr, ntohs(recv_addr.sin_port),
						ntohl(packet.timestamp));
				fflush(stdout);
			}
			else break;

			st = get_usec();

			if (pts * dt <= (st)) break;
		}
		while (1);
	}
	while (1);

	return 0;
}

